C To find the angle between two vectors.
C
C The vectors can be defined by components in real or reciprocal space.
C This subroutine works for any crystal system.
C 
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  The metric tensor is explained on page 22
C  Example 8, page 23 illustrates the actual calculation of angle
C       
      SUBROUTINE MAP_CRYSTAL_ANGLEE(F,G,H,K,L,JR1,HH,KK,LL,JR2,
     &ANG,ADEG,PI)
C
C JR1,JR2 have values of 0 for vectors whose components are defined
C  in real space, and 1 if the components are defined in reciprocal
C  space.
C
C ANG, ANGDEG represent the angle in radians and degrees, respectively
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa
      IMPLICIT NONE
      REAL G(9),F(9),H,K,L,HH,KK,LL,ANG,ADEG,H1,K1,L1,
     &HH1,KK1,LL1,AD,BD,A,B,C,PI,MAP_UTIL_MAG
      INTEGER JR1,JR2
C To calculate the angle, it is necessary for one vector to be
C  defined in real space and the other in reciprocal space. The
C  procedure below first identifies whether real or reciprocal, and
C  then calculates the magnitude of each vector, and then takes
C  a "dot" or scalar product of the two vectors to compute the angle.
C
C Calculate the magnitude of a vector whose components are defined
C  in reciprocal space. Components of vectors in reciprocal space
C  are H, K, L and its corresponding components H1, K1, L1 in real
C  space are obtained by multiplying by the metric tensor F.
C  The square of the magnitudge of the vector is then obtained by
C  H*H1+K*K1+L*L1, the operation carried out by function MAG
C
C
      IF(JR1 .EQ. 0) GOTO 1
        CALL MAP_UTIL_TRANS(F,H,K,L,H1,K1,L1)
        A=MAP_UTIL_MAG(H,K,L,H1,K1,L1)
        A=SQRT(A)
        AD=1.0/A
      GOTO 2
C
1     CALL MAP_UTIL_TRANS(G,H,K,L,H1,K1,L1)
        A=MAP_UTIL_MAG(H,K,L,H1,K1,L1)
        A=SQRT(A)
        AD=1.0/A
C
C
2     IF(JR2 .EQ. 0) GOTO 3
        CALL MAP_UTIL_TRANS(F,HH,KK,LL,HH1,KK1,LL1)
        B=MAP_UTIL_MAG(HH,KK,LL,HH1,KK1,LL1)
        B=SQRT(B)
        BD=1.0/B
C
      GOTO 4
3     CALL MAP_UTIL_TRANS(G,HH,KK,LL,HH1,KK1,LL1)
        B=MAP_UTIL_MAG(HH,KK,LL,HH1,KK1,LL1)
        B=SQRT(B)
        BD=1.0/B
C
C
4     IF(JR1 .EQ. 1 .AND. JR2 .EQ. 0) GOTO 5
      IF(JR1 .EQ. 1 .AND.  JR2 .EQ. 1) GOTO 6
      IF(JR1 .EQ. 0 .AND. JR2 .EQ. 0) GOTO 7
      IF(JR1 .EQ. 0 .AND. JR2 .EQ. 1) GOTO 8
C
5     C=MAP_UTIL_MAG(H,K,L,HH,KK,LL)
        GOTO 9
6     C=MAP_UTIL_MAG(H,K,L,HH1,KK1,LL1)
        GOTO 9
7     C=MAP_UTIL_MAG(H1,K1,L1,HH,KK,LL)
        GOTO 9
8     C=MAP_UTIL_MAG(H,K,L,HH,KK,LL)
C
9     ANG=C/(A*B)
C
C Obtain the angle in radians, then convert into degrees
      ADEG=ACOS(ANG)
      ADEG=ADEG*180.0/PI
C
      RETURN
      END
